// for ISNI
function calc_mod11_2_check_digit(value) {
  // ISO 7064 mod11-2

  let p = 0;
  for (let index = 0; index < value.length; index++) {
    let char = value.charAt(index);
    p = 2 * (p + parseInt(char, 10));
  }
  p %= 11;
  let result = (12 - p) % 11;

  if (result === 10) {
    result = "X";
  }

  // js fun 0 != '0'
  return result.toString();
}

// for GTIN
function calc_mod10_check_digit(value) {
  let calc_checksum = 0;
  let mult;

  // Determine the initial multiplier
  if (value.length % 2 === 1) {
    mult = 3;
  } else {
    mult = 1;
  }

  // Iterate over each character in the value
  for (let i = 0; i < value.length; i++) {
    let char = value.charAt(i);
    calc_checksum += parseInt(char, 10) * mult;

    // Alternate the multiplier between 1 and 3
    if (mult === 1) {
      mult = 3;
    } else {
      mult = 1;
    }
  }

  // Calculate the check digit
  calc_checksum = 10 - (calc_checksum % 10);
  if (calc_checksum === 10) {
    calc_checksum = 0;
  }

  // Return the check digit as a string
  return calc_checksum.toString();
}

// FIXME: these throws should produce user-friendly error messages
export function assert_isrc(isrc) {
  if (!isrc) {
    // No isrc supplied to validate
    throw new Error("No isrc supplied to validate");
  }
  var clean_isrc = isrc.replace(/[^a-z0-9]/gi, "");
  if (!clean_isrc) {
    throw new Error("Wrong Length");
  }
  if (clean_isrc.length != 12) {
    // ISRC must be 12 characters long
    throw new Error("Wrong Length");
  }
  if (!clean_isrc.match(/\d{7}$/)) {
    throw new Error("Final 7 chars must be digits");
  }
  if (!clean_isrc.match(/^[a-z]{2}/i)) {
    throw new Error("First 2 characters of ISRC must be letters");
  }
  return clean_isrc;
}

export function assert_ISNI(value) {
  if (!value) {
    throw new Error("InvalidISNIEmpty");
  }

  value = value.replace(/-/g, "");
  value = value.replace(/\W/g, "");

  if (value.length !== 16) {
    throw new Error("InvalidISNIWrongLength");
  }

  value = value.toUpperCase();
  const orig_check = value.charAt(15);
  value = value.substring(0, 15);

  const calc_check = calc_mod11_2_check_digit(value);

  if (calc_check !== orig_check) {
    throw new Error("InvalidISNIIncorrectCheckSum");
  }

  return value + orig_check;
}

export function assert_barcode(value) {
  if (!value) {
    throw new Error("InvalidEmptyBarcode");
  }

  const length = value.length;
  if (length >= 12 && length <= 14) {
    // UPC or EAN-13 or GTIN-14
    const check_digit = value.slice(-1);
    const calc_checksum = calc_mod10_check_digit(value.slice(0, -1));

    if (calc_checksum !== check_digit) {
      throw new Error(
        "InvalidBarcodeIncorrectCheckSum for: |" +
          value +
          "| (remove any whitespace before or after barcode)",
      );
    }
  } else {
    throw new Error(
      "InvalidBarcodeIncorrectNumberOfDigits for: |" +
        value +
        "| (remove any whitespace before or after barcode)",
    );
  }

  return value;
}
