package state51::AnyEvent;
use strict;
use warnings;
use AnyEvent;
use Exporter qw/ import /;
use vars qw/ @EXPORT_OK /;

our @EXPORT_OK = qw/
    FULL_ASYNC
    run_loop_till_idle
/;

sub FULL_ASYNC () {
    return 1 if (grep { m{^Twiggy/Server.pm} } keys %INC);
    return 1 if $AnyEvent::CondVar::Base::WAITING;
    return;
}

our $RECURSE = 0;
sub run_loop_till_idle {
    local $RECURSE = $RECURSE + 1;
    return if $RECURSE or FULL_ASYNC;
    my $cv = AnyEvent->condvar;
    my $idle = AnyEvent->idle(cb => sub { $cv->send });
    $cv->recv;
    $cv = AnyEvent->condvar;
    $idle = AnyEvent->timer(after => 2, cb => sub { $cv->send });
    $cv->recv;
    return;
}

1;

=head1 NAME

state51::AnyEvent - Some helper functions for writing AnyEvent code in state51

=head1 EXPORTABLE FUNCTIONS

=head2 FULL_ASYNC

Tells you if you are in a fully asynchronous environment. If this is true
then you B<CANNOT> block on convars (i.e. you can only call C<< $cv->recv >>
on vars you know have been has the C<< ->send >> method called on them.

=head1 run_loop_till_idle

Run the event loop to cause any pending events to be flushed.

Call this regularly when not fully async to cause event loop
processing to happen (but to avoid blocking on a condvar)..

=cut

