package state51::Mixin::HasCache;
use Moose::Role;
use Cache::Null;
use Cache::Memcached;
use Moose::Util::TypeConstraints;
use MooseX::Storage ();
use namespace::autoclean;

has _cache => (
    isa => duck_type([qw/ get set /]),
    is => 'ro',
    lazy_build => 1,
    handles => {
        cache_get => 'get',
        cache_set => 'set',
        cache_delete => 'remove',
    },
    traits => ['DoNotSerialize'],
);

{
    my $cache;

    sub _build__cache {
        my $self = shift;
        return $cache if $cache;
        $cache = $self->_build_cache_firsttime;
    }

    sub _build_cache_firsttime {
        if ($ENV{MEMCACHE_SERVERS}) {
            $cache = eval {
                Cache::Memcached->new({
                    servers => [ split /,/, $ENV{MEMCACHE_SERVERS} ],
                    no_rehash => 1,
                });
            };
            warn("Error setting up memcache backend: $@") if $@;
        }
        return $cache ||= Cache::Null->new;
    }
}

1;

