package state51::RabbitMQ::HasAChannel;
use state51::Moose::Role;
use state51::RabbitMQ;
use state51::AnyEvent qw/ FULL_ASYNC /;
use MooseX::Types::Moose qw/ Object /;
use state51::AnyEvent qw/ run_loop_till_idle /;
use Scalar::Util qw/ weaken /;

has channel => (
    is => 'ro',
    isa => Object,
    writer => 'set_channel',
    predicate => '_has_channel',
    clearer => '_clear_channel',
);

has _building_channel => (
    is => 'rw',
    isa => Bool,
    default => 0,
);

sub BUILD {}
after BUILD => sub {
    my $self = shift;
    state51::RabbitMQ->instance->mq(sub {
        my $mq = shift;
        $self->build_channel($mq);
    })
};

sub build_channel {
    my ($self, $mq) = @_;
    return $self->channel if $self->_has_channel;
    $mq ||= state51::RabbitMQ->instance->mq();
    return unless $mq;
    return if $self->_building_channel;
    $self->_building_channel(1);
    my $weak_self = $self;
    weaken($weak_self);
    $mq->open_channel(
        on_success => sub {
            my $channel = shift;
            return unless $weak_self;
            $weak_self->_building_channel(0);
            $weak_self->set_channel($channel);
        },
        on_failure => sub {
            return unless $weak_self;
            $weak_self->_building_channel(0);
            $weak_self->_clear_channel;
        },
        on_close   => sub {
            return unless $weak_self;
            $weak_self->_building_channel(0);
            $weak_self->_clear_channel;
            my $method_frame = shift->method_frame;
            #die $method_frame->reply_code, $method_frame->reply_text;
        },
    );
    run_loop_till_idle();
}

1;

