package state51::Script::MessageJobs::SharedWorkerPools;
use state51::Moose;
use state51::Script::MessageJobs::WorkerPool;
use MooseX::Types::Moose qw/ HashRef Str ArrayRef Int Num /;
use Moose::Util::TypeConstraints;
use AnyEvent;
use Data::OptList;

class_type 'state51::Script::MessageJobs::WorkerPool';

has pools => (
    isa => ArrayRef[Str],
    is  => 'ro',
    required => 1,
);

has '_pools' => (
    isa => HashRef['state51::Script::MessageJobs::WorkerPool'],
    is => 'ro',
    lazy => 1,
    default => sub {
        my ($self) = @_;
        return {
            "shared_worker" => state51::Script::MessageJobs::WorkerPool->new(
                queue_names => $self->pools,
                exit_when_complete => 0,
                max_workers => $self->workers,
                number_of_jobs_before_terminate => $self->number_of_jobs_before_terminate,
            )
        };
    },
    traits => ['Hash'],
    handles => {
        get_pool_by_name => 'get',
    },
);

has keep_running_file => (
    isa => Str,
    is  => 'ro',
    documentation => "The name of a file which when removed I will terminate.",
);

has workers => (
    isa => Int,
    is => 'rw',
    required => 1,
    default => 2,
);

has number_of_jobs_before_terminate => (
    isa => Int,
    is => 'rw',
    required => 1,
    default => 50,
);

sub run {
    my $self = shift;
    my @cvs;

    if ($self->keep_running_file) {
        my $w; $w = AnyEvent->timer(
            after => 5,
            interval => 5,
            cb => sub {
                if (! -e $self->keep_running_file) {
                    $_->stop(1) for values %{$self->_pools};
                    $w=undef;
                }
            },
        );
    }

    push(@cvs, $_->run) for values %{$self->_pools};
    $_->recv for @cvs;

    return;
}

with qw/ state51::Script::Role::ScriptBase /;

has '+configfile' => (
    is => 'ro',
    isa => 'Str',
    lazy => 1,
    coerce => 1,
    predicate => 'has_configfile',
);

__PACKAGE__->meta->make_immutable;
1;

