package state51::Script::Role::ScriptBase;

use Moose::Role;
use File::HomeDir;
use Getopt::Long::Descriptive; # Force GLD as we override bits..
use YAML qw/LoadFile/;
use Path::Class;

use MooseX::Storage ();
use MooseX::Types::Moose qw/ ArrayRef Bool /;
use namespace::clean -except => [qw/ meta /];

with qw/
    MooseX::Getopt
    MooseX::ConfigFromFile
/;

has 'ARGV' => (
    is => 'rw',
    traits => [qw/ NoGetopt DoNotSerialize /],
    isa => ArrayRef,
);
has 'extra_argv' => (
    is => 'rw',
    traits => [qw/ NoGetopt DoNotSerialize /],
    isa => ArrayRef,
);

requires 'run';

sub get_config_from_file {
    my ($class, $file) = @_;
    return LoadFile($file) if (-r $file);
    return {};
}

has 'configfile' => (
    default => file(File::HomeDir->my_data(), ".s51platform.yml")->stringify,
    is => 'bare',
    traits => [qw/ DoNotSerialize /],
);

sub postfix_useage {}

around '_gld_spec' => sub {
    my $orig = shift;
    my $class = shift;
    my ($options, $name_to_init_arg) = $class->$orig(@_);
    push(@$options, ([ "" ], map { [ $_ ] } split /\r?\n/, $class->postfix_useage || ''));
    return ($options, $name_to_init_arg);
};

1;

__END__

