package state51::WriteToSyslogIfNonInteractive;
use strict;
use warnings;
use IO::Handle;

my $logger_fh;
my $logger_name;
sub import {
    shift();
    $logger_name = shift() || $0;
    _SETUP();
}

sub _SETUP {
    return unless $logger_name;
    return if -t STDIN;
    open($logger_fh, "|/usr/bin/logger -t $logger_name")
        or warn "Could not open output to logger: $!";
    return unless $logger_fh;
    $logger_fh->autoflush(1);
    open( STDOUT, '>&='. $logger_fh->fileno )
            or warn("Can't re-open stdout to logger: $!");
    open( STDERR, '>&='. $logger_fh->fileno )
            or warn("Can't re-open stderr to logger: $!");
}

sub _TEARDOWN {
    return unless $logger_name;
    return if -t STDIN;
    close($logger_fh) if $logger_fh;
    undef $logger_fh;
    close(STDOUT);
    close(STDERR);
}

BEGIN { _SETUP() }
END { _TEARDOWN() }

1;

