use strict;
use warnings;
use Test::More;
use Test::Exception;

use state51::Types qw/
    WhitespaceTrimmedStr
    MaybeWhitespaceTrimmedStr
    ExplicitStatus
    FileStatus
    CurrencyName
    DDEXMessageType
    AspectRatio
    ISNI
/;
use state51::Validate;

ok is_MaybeWhitespaceTrimmedStr(undef);
ok !is_WhitespaceTrimmedStr(undef);

ok is_MaybeWhitespaceTrimmedStr("foo");
ok is_WhitespaceTrimmedStr("foo");

ok !is_MaybeWhitespaceTrimmedStr(""); # Note the Maybe coerces to undef
ok is_WhitespaceTrimmedStr("");

is to_MaybeWhitespaceTrimmedStr(""), undef;
is to_WhitespaceTrimmedStr(""), "";

ok is_ExplicitStatus('yes');
ok !is_ExplicitStatus('naaarrf');

ok is_FileStatus('pending');
ok !is_FileStatus('moo');

ok is_CurrencyName('gbp');
ok !is_CurrencyName('dfjkweqjhfjn');

ok is_DDEXMessageType('ern:NewReleaseMessage');
ok !is_DDEXMessageType('foobar:NewReleaseMessage');

ok is_AspectRatio('16:9');
ok is_AspectRatio('16.0:9.0');
ok is_AspectRatio('1.234:6.789');
ok !is_AspectRatio('1.234/6.789');
ok !is_AspectRatio(':');
ok !is_AspectRatio('');

throws_ok{state51::Validate::assert_ISNI('2222222222')} qr/InvalidISNIWrongLength/, 'caught wrong length';
throws_ok{state51::Validate::assert_ISNI('000000012103268X')} qr/InvalidISNIIncorrectCheckSum/, 'caught bad checksum';
lives_ok{state51::Validate::assert_ISNI('0000000121032683')} 'Valid ISNI is valid';
lives_ok{state51::Validate::assert_ISNI('1422458635730476')} 'Valid ISNI is valid';

done_testing;
