#!/usr/local/bin/perl
use FindBin qw/$Bin/;
use lib "$Bin/../lib";

use strict;
use warnings;

use Test::More;

use Try::Tiny;
use state51::Validate;

my $error;

# not enough chars
$error = 'reset';
try { state51::Validate::assert_isrc('FR-Z03-91-0123'); }
catch {$error = $_};
like($error, qr/Wrong Length/,'not enough chars');

# too many chars
$error = 'reset';
try { state51::Validate::assert_isrc('FR-Z03-91-012322'); }
catch {$error = $_};
like($error, qr/Wrong Length/,'too many chars');

# this one is ok!
$error = 'reset';
try { state51::Validate::assert_isrc('FR-Z03-91-01232'); }
catch {$error = $_};
cmp_ok($error, 'eq', 'reset', 'fine with -');

# this one is ok!
$error = 'reset';
try { state51::Validate::assert_isrc('FRZ039101232'); }
catch {$error = $_};
cmp_ok($error, 'eq', 'reset', 'fine without -');

# this one is ok!
$error = 'reset';
try { state51::Validate::assert_isrc(' FR Z039 101 23 2'); }
catch  {$error = $_};
cmp_ok($error, 'eq', 'reset', 'crazy random whitespace');

# bad chars
$error = 'reset';
try { state51::Validate::assert_isrc('---039101111'); }
catch  {$error = $_};
like($error, qr/Wrong Length/,"- doesn't count");

# bad chars
$error = 'reset';
try { state51::Validate::assert_isrc('___039101111'); }
catch  {$error = $_};
like($error, qr/Wrong Length/,"_ doesn't count");

# bad year
$error = 'reset';
try { state51::Validate::assert_isrc('FR-Z03-Z1-01232'); }
catch  {$error = $_};
like($error, qr/Final 7 chars must be digits/,'final 7 chars must be digits');

# bad designation code
$error = 'reset';
try { state51::Validate::assert_isrc('FR-Z03-11-0123Z'); }
catch {$error = $_};
like($error, qr/Final 7 chars must be digits/,'final 7 chars must be digits');

# UPC in the isrc field
$error = 'reset';
try { state51::Validate::assert_isrc('824833005168'); }
catch  {$error = $_};
like($error, qr/First 2 characters/,"first 2 chars are letters");

# Digit in the second slot. Not allowed.
$error = 'reset';
try { state51::Validate::assert_isrc('D2DM20600034'); }
catch  {$error = $_};
like($error, qr/First 2 characters/,"both first 2 chars are letters");

# Tunecore country code. IFPI decided to allow these specially
$error = 'reset';
try { state51::Validate::assert_isrc('TCDM20600034'); }
catch  {$error = $_};
cmp_ok($error, 'eq', 'reset', 'Silly special TuneCore code');

done_testing();
