use strict;
use warnings;
use Test::More;
BEGIN { $ENV{FILE_CACHE_DIR} = '/tmp' }
use vars qw/ $DEBUG @DONE @TODO $EXIT $DOTERM $JOBDIE /;

local $DEBUG = 0;

local @DONE;
local @TODO;
local $EXIT;

local $DOTERM;
local $JOBDIE;
{
    package TestClass;
    use state51::Moose;

    sub cooldown_type {
        return 'foo';
    }

    with 'state51::Script::Role::WorkPool::Worker';

    sub do_job {
        my ($self, $job) = @_;
        push(@::DONE, $job);
        if ($::DOTERM) {
            ::is ref($SIG{TERM}), 'CODE', 'Overridden SIG{TERM}'
                and $SIG{TERM}->();
        }
        if ($::JOBDIE) {
            die("Test job die");
        }
    }

    sub get_job {
        my ($self) = @_;
        my $job = shift @::TODO;
        [$job];
    }

    sub _exit {
        my ($self, $exit) = @_;
        $::EXIT = $exit;
    }

    around log_message => sub {
        my ($orig, $self, @args) = @_;
        $self->$orig(@args) if $::DEBUG;
    };

    has '+verbose' => ( default => $::DEBUG ? 1 : 0 );
}

TestClass->new->run;
is $EXIT, 1, 'Had nothing to do';
undef $EXIT;

@TODO = ('job');
{
    my $i = TestClass->new(number_of_jobs_before_terminate => 10);
    $i->run;
    is $i->number_of_jobs_before_terminate, 9;
    is $i->number_of_jobs_run, 1;
}
is scalar(@TODO), 0;
is $EXIT, 0, 'did work';
is_deeply \@DONE, ['job'];
@DONE = ();

@TODO = ('job', 'job');
{
    my $i = TestClass->new(number_of_jobs_before_terminate => 1);
    $i->run;
    is $i->number_of_jobs_before_terminate, 0;
    is $i->number_of_jobs_run, 1;
}
is scalar(@TODO), 1, 'work left';
is $EXIT, 0, 'did work';
is_deeply \@DONE, ['job'];
@DONE = ();

@TODO = ('job', 'job');
$DOTERM = 1;
{
    my $i = TestClass->new(number_of_jobs_before_terminate => 10);
    $i->run;
    is $i->number_of_jobs_before_terminate, 9;
    is $i->number_of_jobs_run, 1;
}
is scalar(@TODO), 1, 'work left';
is $EXIT, 0, 'did work';
is_deeply \@DONE, ['job'];
$DOTERM = 0;
@DONE = ();

@TODO = ('job', 'job');
$JOBDIE = 1;
{
    my $i = TestClass->new(number_of_jobs_before_terminate => 10);
    $i->run;
    is $i->number_of_jobs_before_terminate, 9;
    is $i->number_of_jobs_run, 1;
}
is scalar(@TODO), 1, 'work left';
is $EXIT, 0, 'did work';
is_deeply \@DONE, ['job'];
$DOTERM = 0;
@DONE = ();

done_testing;

